#include "SimpleAnalysisFramework/AnalysisClass.h"
#include <array>
#include <TRandom.h>
#include <TFile.h>
#include <TH2.h>
#include <TF1.h>
#include "TLorentzVector.h"
#include "TMath.h"
#include <vector>
#include "TTree.h"
#include "TString.h"
#include "TROOT.h"
#include <iostream>
#include <fstream>

DefineAnalysis(SS3Lstrong2020)

static float SS3Lstr_GetSFOSmass(AnalysisObjects& leptons)
{
    float c(0);
    int nLep = leptons.size();
    switch(nLep){
        case 2:
            c = 0;
            break;
        case 3:
      if ( leptons[0].charge() != leptons[1].charge() && leptons[0].type()==leptons[1].type() ) {
        c=(leptons[0]+leptons[1]).M();
      }
      if ( leptons[0].charge() != leptons[2].charge() && leptons[0].type()==leptons[2].type() ) {
        float calt=(leptons[0]+leptons[2]).M();
        if (fabs(calt-91)<fabs(c-91)) c=calt;
      }
      if ( leptons[2].charge() != leptons[1].charge() && leptons[2].type()==leptons[1].type() ) {
        float calt=(leptons[1]+leptons[2]).M();
        if (fabs(calt-91)<fabs(c-91)) c=calt;
      }
      break;
        case 4:
      if ( leptons[0].charge() != leptons[1].charge() && leptons[0].type()==leptons[1].type() ) {
        float calt=(leptons[0]+leptons[1]).M();
        if (fabs(calt-91)<fabs(c-91)) c=calt;
      }
      if ( leptons[0].charge() != leptons[2].charge() && leptons[0].type()==leptons[2].type() ) {
        float calt=(leptons[0]+leptons[2]).M();
        if (fabs(calt-91)<fabs(c-91)) c=calt;
      }
      if ( leptons[0].charge() != leptons[3].charge() && leptons[0].type()==leptons[3].type() ) {
        float calt=(leptons[0]+leptons[3]).M();
        if (fabs(calt-91)<fabs(c-91)) c=calt;
      }
      if ( leptons[1].charge() != leptons[2].charge() && leptons[1].type()==leptons[2].type() ) {
        float calt=(leptons[1]+leptons[2]).M();
        if (fabs(calt-91)<fabs(c-91)) c=calt;
      }
      if ( leptons[1].charge() != leptons[3].charge() && leptons[1].type()==leptons[3].type() ) {
        float calt=(leptons[1]+leptons[3]).M();
        if (fabs(calt-91)<fabs(c-91)) c=calt;
      }
      if ( leptons[2].charge() != leptons[3].charge() && leptons[2].type()==leptons[3].type() ) {
        float calt=(leptons[2]+leptons[3]).M();
        if (fabs(calt-91)<fabs(c-91)) c=calt;
      }
      break;
    
    default: //FIXME? doesn't handle more than 4 leptons?
      break;
    }
    return c;
}

                                                                                                            
static float SS3Lstr_getDeltaRLepLep(AnalysisObjects& leptons, bool ForSS, int Flag){
    float dR(-99.), dR_min(99.), dR_max(-99.);
    if(leptons.size()==0) return dR;
    if(leptons.size() < 2) return dR;
    
    int charges_size = leptons.size();  
    if(!ForSS) {dR = leptons.at(0).DeltaR(leptons.at(1)); return dR;}
    else{
      for(int i=0; i<charges_size-1; i++){
    for(int j=i+1; j<charges_size; j++){
      if(leptons[i].charge()*leptons[j].charge()>0){
        dR = leptons.at(i).DeltaR(leptons.at(j));
        if (Flag == 0) { break; return dR;}
        else {
          if (dR<dR_min) dR_min = dR;
          if (dR>dR_max) dR_max = dR;
        }
      }
    }
      }
      if (Flag == 0 ) return dR;
      else if (Flag == 1 ) return  dR_max;
      else return dR_min;
    }
}

    
static bool SS3Lstr_SSleptons(AnalysisObjects& leptons)
{
  bool passCharges(false);
  unsigned int LepSize = leptons.size();
  if( LepSize<2 ) {
    passCharges=false;
  }
  if(LepSize==2){
    passCharges = leptons[0].charge()*leptons[1].charge()>0;
  }
  if(LepSize>2){
    passCharges = true;
  }
  return passCharges;
}



static float SS3Lstr_getDeltaRLepJet(AnalysisObjects& leptons, AnalysisObjects& jets, int N, bool useLep){
    float minDR = 99., dR=-99., maxDR = -100.0;
        bool getMax=false;

    if (!useLep){

        if (N>=0) {

            if (int(leptons.size())<(N+1)) return -99.;
            for (unsigned int j=0;j<jets.size();j++){

          if (leptons.at(N).DrEtaPhi(jets.at(j))<minDR && jets.at(j).Pt()>25){
        minDR=leptons.at(N).DrEtaPhi(jets.at(j));
        
          }
          if (leptons.at(N).DrEtaPhi(jets.at(j))>maxDR && jets.at(j).Pt()>25){
        maxDR=leptons.at(N).DrEtaPhi(jets.at(j));

          }
            }
        }
        else {
            for (int j=0;j<int(jets.size());j++){
                for (int i=0;i<int(leptons.size());i++){
          dR = leptons.at(i).DrEtaPhi(jets.at(j));  
          if (dR<minDR) minDR = dR;
          if (dR>maxDR) maxDR = dR;

                }
            }
        }
    }
    else {

        if (int(jets.size())<(N+1)) return -99.;
        for (unsigned int j=0;j<leptons.size();j++){
      if (jets.at(N).DrEtaPhi(leptons.at(j))<minDR){
        minDR=leptons.at(j).DrEtaPhi(jets.at(N));
      }
      if (jets.at(N).DrEtaPhi(leptons.at(j))>maxDR){
        maxDR=leptons.at(j).DrEtaPhi(jets.at(N));
      }
        }
    }
    if (!getMax) return minDR;
    else return maxDR;
}

static float SS3Lstr_getdPhiLLmet(AnalysisObjects& leptons, AnalysisObject metVec){
     float dPhi(-99.);
     if(leptons.size()<2){ 
         return dPhi;}
     else{
         dPhi=fabs(metVec.DeltaPhi(leptons[0]+leptons[1]));
     return dPhi;
   }
}

static int SS3Lstr_isSSLepPtCut(AnalysisObjects& leptons){
  int isAbove30 = 0;
  int isSSpair = 0;
  for(unsigned int i(0); i<leptons.size(); i++){
    for(unsigned int j(0); j<leptons.size(); j++){
      if(i==j || isSSpair==1)
    continue;
      if(leptons[i].charge()==leptons[j].charge())
    isSSpair = 1;
      if(isSSpair==1 && leptons.at(i).Pt()>30 && leptons.at(j).Pt()>30)
    isAbove30=1;
    }
  }
  return isAbove30;
}


/**********************************init***************************************/
//VRWZ4j,VRWZ6j,VRTTV,VRTTW,VRTTW3j
void SS3Lstrong2020::Init() {
addRegion("preselection");
addRegion("CRWZ2j");
addRegion("VRTTV");
addRegion("VRWZ4j");
addRegion("VRWZ6j");
addRegion("VRTTW");
addRegion("VRTTW3j");

addRegion("RpcGGwz1");
addRegion("RpcGGwz2");
addRegion("RpcGGwz3");
addRegion("RpcGGslep1");
addRegion("RpcGGslep2");
addRegion("RpcGGslep3");
addRegion("RpvUDD1b");
addRegion("RpvUDD2b");
addRegion("RpvUDDge2b");
addRegion("RpvUDDge3b");
addRegion("RpvLQD");
addRegion("RpcSSslep1");
addRegion("RpcSSslep2");
addRegion("RpcSSslep3");
addRegion("RpcSSslep4");
addRegion("RpcSSslep1_Excl");
addRegion("RpcSSwz1");
addRegion("RpcSSwz2");
addRegion("RpcSSwz3");
addRegion("RpcSSwz4");

addRegion("InclusiveRpvUDD1b");
addHistogram("nSigLep",10,0,10);
addHistogram("met",25,0,500);
addHistogram("ht_jet",20,0,3000);
addHistogram("Pt_l",20,0,1000);
addHistogram("Pt_subl",25,0,500);
addHistogram("ht_bjet",20,0,3000);
addHistogram("ht_lept",20,0,1000);
addHistogram("dRl1j",10,0,5);
addHistogram("dRll",10,0,5);
addHistogram("mSFOS",20,0,200);
addHistogram("mll",20,0,1000);
addHistogram("meff",20,0,4000);
addHistogram("mjj",20,0,1000);
addHistogram("Pt_jet",25,0,500);
addHistogram("Pt_subjet",25,0,500);
addHistogram("Pt_Bjet",20,0,1000);
addHistogram("nJets40",10,0,10);
addHistogram("nJets25",12,0,12);
addHistogram("nBjets20",10,0,10);
addHistogram("mt_min",20,0,1000);
addHistogram("dPhiLLmet",20,0,20);
}

void SS3Lstrong2020::ProcessEvent(AnalysisEvent *event){

	auto baselineElectrons = filterCrack(event->getElectrons(10, 2.47, ELooseBLLH|ED0Sigma5|EZ05mm));
	auto baselineMuons = event->getMuons(10, 2.5, MuMedium|MuZ05mm|MuQoPSignificance|MuNotCosmic);

	auto Jets = event->getJets(20, 2.8,JVTTight|LooseBadJet);
	auto overlapBjet = filterObjectsRange(Jets,0,100,-99,99,BTag70DL1r);

//********overlap removal******************//
	auto radiusCalcJet  = [] (const AnalysisObject& , const AnalysisObject& muon){ return std::min(0.4, 0.1 + 9.6/muon.Pt()); };
	auto radiusCalcMuon = [] (const AnalysisObject& muon, const AnalysisObject& ){ return std::min(0.4, 0.1 + 9.6/muon.Pt()); };
	auto radiusCalcElec = [] (const AnalysisObject& elec, const AnalysisObject& ) { return std::min(0.4, 0.1 + 9.6/elec.Pt()); };

	baselineElectrons=overlapRemoval(baselineElectrons, baselineElectrons , 0.05);
	baselineElectrons=overlapRemoval(baselineElectrons, baselineMuons , 0.01);
	Jets = overlapRemoval(Jets, baselineElectrons, 0.2, NOT(BTag70DL1r));
	baselineElectrons = overlapRemoval(baselineElectrons, Jets, radiusCalcElec);

	Jets = overlapRemoval(Jets,baselineMuons,0.4,LessThan3Tracks);
	baselineMuons = overlapRemoval(baselineMuons, Jets, radiusCalcMuon);
	baselineElectrons=overlapRemoval(baselineElectrons, overlapBjet , 0.2);

	auto metVec = event->getMET();
	double metSignificance =event->getMETSignificance();
	double met = metVec.Et();
    auto signalElectrons = filterObjects(baselineElectrons, (met > 250) ? 10 : 20, 2.0,EMediumLH|EIsoFCTight);
    auto signalMuons = filterObjects(baselineMuons,(met > 250) ? 10 : 20, 2.5, MuIsoFCTightTrackOnly|MuD0Sigma3);

	auto Bjets = filterObjects(Jets,20,2.5,BTag70DL1r);

	auto baseLeptons = baselineMuons+baselineElectrons;
	auto signalLeptons = signalElectrons+signalMuons;
	int nBjets = Bjets.size();
	float Pt_Bjet=0;
	if(nBjets>0)Pt_Bjet=Bjets[0].Pt();

    int nJets20  = countObjects(Jets, 20.);
	int nJets25  = countObjects(Jets, 25.); 
    int nJets35  = countObjects(Jets, 35.);
	int nJets40  = countObjects(Jets, 40.);
	int nJets50  = countObjects(Jets, 50.);
    int nJets60  = countObjects(Jets, 60.);
	int nJet=Jets.size();
	int nSigLep=signalLeptons.size();
	int NlepBL=baseLeptons.size();
	int nBjets20 = countObjects(Bjets,20);
	double meff = sumObjectsPt(Jets) + sumObjectsPt(signalLeptons) + met;

	float dRll =-99;
	float dRl1j=-99;
	float mjj(0),mll(0);
	float ht_lept=sumObjectsPt(signalLeptons);
	float ht_jet=sumObjectsPt(Jets);
	float ht_bjet=sumObjectsPt(Bjets);
/*********************************select********************************/
	if (signalLeptons.size()<2) return;
	if (baseLeptons.size()<2) return;
	if (Jets.size()<1) return;
	bool isee = false;
	bool ismm = false;
	bool isem = false;
if(signalLeptons.size()>1){
  auto lep1 = signalLeptons[0];
	if( lep1.type() == ELECTRON ){
		for(int i =1;i<signalLeptons.size();i++){
			if(signalLeptons[i].type() == ELECTRON&&signalLeptons[i].charge()==lep1.charge())isee=true;
		}
	}
	if( lep1.type() == MUON ){
    for(int i =1;i<signalLeptons.size();i++){
      if(signalLeptons[i].type() == MUON&&signalLeptons[i].charge()==lep1.charge())ismm=true;
		}
   }
  if( lep1.type() == MUON ){
    for(int i =0;i<signalLeptons.size();i++){
      if(signalLeptons[i].type() == ELECTRON&&signalLeptons[i].charge()==lep1.charge())isem=true;
    }
   }
	 if( lep1.type() == ELECTRON ){
    for(int i =0;i<signalLeptons.size();i++){
		  if(signalLeptons[i].type() == MUON&&signalLeptons[i].charge()==lep1.charge())isem=true;
		}
   }
}

	bool SameSign = SS3Lstr_SSleptons(signalLeptons);
	if(!SameSign)return;
	float mSFOS(0),Pt_jet(0),Pt_subjet(0),Pt_thirdl(0),mt_min(0),dPhiLLmet(-99.);
    bool miupair=signalMuons.size()==2&&(signalMuons[0].charge()*signalMuons[1].charge()>0);
    bool emiupair=signalMuons.size()==1&&signalElectrons.size()==1&&(signalMuons[0].charge()*signalElectrons[0].charge()>0);

	mSFOS=SS3Lstr_GetSFOSmass(signalLeptons);
    dPhiLLmet = SS3Lstr_getdPhiLLmet(signalLeptons, metVec);
	int isSS30 = SS3Lstr_isSSLepPtCut(signalLeptons);
	Pt_jet=Jets[0].Pt();
	if(Jets.size()>1)Pt_subjet=Jets[1].Pt();
	if(signalLeptons.size()>2)Pt_thirdl=signalLeptons[2].Pt();

    float Pt_l=signalLeptons[0].Pt();
	float Pt_subl=signalLeptons[1].Pt();
	mjj = Jets.size()>1 ? (Jets.at(0) + Jets.at(1)).M() : 0.;
	mll = (signalLeptons.at(0) + signalLeptons.at(1)).M();

    mt_min=calcMTmin(signalLeptons, metVec);

	dRll  = SS3Lstr_getDeltaRLepLep(signalLeptons, false, 0);
	dRl1j = SS3Lstr_getDeltaRLepJet(signalLeptons, Jets, 0, false);
	
	bool preselection=nBjets20>0;
    bool BlindingVeto=!(nBjets20>=3&&nJets40>=4) && !(nBjets20>=1 && nJets40>=4 && met>=130) && !(nBjets20==0 && nJets40>=3 && met>=130) && !(nBjets20==0 && nJets40>=5);
    bool Zevent=mSFOS>81&&mSFOS<101;
    bool CRWZ2j=nSigLep==3 &&NlepBL==3 && Pt_l>15 && Pt_subl>15 && nBjets20==0 && nJets25>=2 && met>30 && met<150 && meff<1500 && ht_lept>130 && mSFOS>81 && mSFOS<101&&BlindingVeto;
	bool VRWZ4j=nSigLep==3 && NlepBL==3 && nBjets20==0 && nJets25>=4&&meff>600&&meff<1500&&met>30&&met<250&&mSFOS>81&&mSFOS<101&&BlindingVeto;
    bool VRWZ6j=nSigLep==3 && NlepBL==3 && nBjets20==0 && nJets25>=6&&meff>400&&meff<1500&&met>30&&met<250&&mSFOS>81&&mSFOS<101&&BlindingVeto;
    bool VRTTV=SameSign&&NlepBL>=2&&nSigLep>=2&&nBjets20>=1&&nJets40>=3&&meff>600&&meff<1500&&met>30&&met<250&&isSS30&&dRl1j>1.1&&(ht_bjet/ht_jet>0.4)&&(met/meff)>0.1&&BlindingVeto;
    bool VRTTW=miupair && NlepBL==2 && nSigLep==2 && nBjets20>=2 && meff<1500 && met>30 && met<250 && Pt_l>40 && Pt_subl>25&&BlindingVeto;
    bool VRTTW3j=emiupair && NlepBL==2 && nSigLep==2 && nBjets20>=2 && nJets25>=3 && meff<1500 && met>30 && met<250 && Pt_l>25 && Pt_subl>25&&BlindingVeto;

    bool RpcGGwz1=nSigLep>=2&&nBjets20==0&&nJets40>=6&&met>=150&&meff>=2100;
    bool RpcGGwz2=nSigLep>=2&&nBjets20==0&&nJets40>=6&&met>=190&&meff>=1300&&dPhiLLmet>0.8;
    bool RpcGGwz3=nSigLep>=2&&nBjets20==0&&nJets25>=6&&met>=200&&metSignificance>6&&(meff/ht_lept)>8&&dPhiLLmet>0.2&&NlepBL>=3;

    bool RpcGGslep1=nSigLep>=3&&nJets40>=4&&nBjets20==0 && !Zevent &&(met/ht_jet)>=0.4&&(met/ht_lept)>=1.4&&Pt_subl>=30;
    bool RpcGGslep2=nSigLep>=3&&nJets40>=4&&nBjets20==0 && !Zevent &&(met/ht_jet)>=0.3&&dPhiLLmet>0.7&&Pt_subl>=70&&met>=150;
    bool RpcGGslep3=nSigLep>=3&&nJets40>=4&&nBjets20==0 && !Zevent &&met>=100&&ht_jet>=1200;

    bool RpvUDD1b=nSigLep==2&&nJets50 >=6 && nBjets20==1 && ht_jet>=1600;
    bool RpvUDD2b=nSigLep==2&&nJets25 >=2 && nBjets20==2 && ht_jet>=1700;
    bool RpvUDDge2b=nSigLep==2&&nJets50 >=5 && nBjets20>=2 && ht_jet>=1600;
    bool RpvUDDge3b=nSigLep==2&&nJets50 >=4 && nBjets20>=3 && meff>=1600;

    bool InclusiveRpvUDD1b=nSigLep==2&&nJets50 >=6 && ht_jet>=1600;

    bool RpvLQD=nSigLep==2&&nJets50 >=5&&meff>=2600;

    bool RpcSSslep1=nSigLep==3&&NlepBL==3&&Pt_l>40&&Pt_subl>40&&Pt_thirdl>40&&nBjets20==0 &&nJets60>=2&&met>=200&&meff>=2000&&dPhiLLmet>=0.3&&dRll>=0.5&&(mSFOS<81||mSFOS>101);
    bool RpcSSslep1_Excl=nSigLep==3&&NlepBL==3&&Pt_l>40&&Pt_subl>40&&Pt_thirdl>40&&nBjets20==0 &&nJets60>=2&&met>=200&&meff>=1000&&dPhiLLmet>=0.3&&dRll>=0.5&&(mSFOS<81||mSFOS>101);
    bool RpcSSslep2=nSigLep==3&&NlepBL==3&&Pt_l>40&&Pt_subl>40&&Pt_thirdl>40&&nBjets20==0 &&nJets60>=2&&met>=200&&meff>=1000&&(met/ht_lept)>=0.7&&dPhiLLmet>=0.5&&dRll>=0.2&&(mSFOS<81||mSFOS>101);
    bool RpcSSslep3=nSigLep==3&&NlepBL==3&&Pt_l<=60&&nBjets20==0 &&nJets60>=2&&nJets25>=3&&met>=100&&meff>=600&&(ht_lept/ht_jet)<=0.6&&dPhiLLmet>=1.4&&(mSFOS<81||mSFOS>101);
    bool RpcSSslep4=nSigLep==3&&NlepBL==3&&Pt_l>30&&Pt_subl>30&&Pt_thirdl>30&&nBjets20==0 &&nJets60>=2&&nJets25>=3&&met>=100&&meff>=700&&(met/ht_lept)>=0.7&&(ht_lept/ht_jet)<=0.6&&dPhiLLmet>=1.4&&(mSFOS<81||mSFOS>101);

    bool RpcSSwz1= nSigLep>=3&& !(mSFOS>81 && mSFOS<101)&& nBjets20==0&& nJets25>=4&& met/meff >=0.200&& ht_lept/ht_jet <=0.200;
    bool RpcSSwz2= nSigLep>=3&& (mSFOS>81 && mSFOS<101) && nBjets20==0&& meff>=800&& met>=150&& nJets25>=6&& met/ht_lept >=1.200&& ht_lept/ht_jet <=0.300;
    bool RpcSSwz3= nSigLep>=3&& (mSFOS>81 && mSFOS<101) && nBjets20==0&& meff>=900&& met>=200&& nJets40>=5&& met/ht_lept >=1.100&& ht_lept/ht_jet <=0.400;
    bool RpcSSwz4= nSigLep>=3&& nBjets20==0&& meff>=1500&& met>=250&& nJets40>=5&& met/ht_lept >=0.300&& ht_lept/ht_jet <=0.700;

	


	if(CRWZ2j)accept("CRWZ2j");
	if(VRTTV)accept("VRTTV");
	if(VRWZ4j)accept("VRWZ4j");
	if(VRWZ6j)accept("VRWZ6j");
    if(VRTTW)accept("VRTTW");
    if(VRTTW3j)accept("VRTTW3j");
    if(RpcGGwz1)accept("RpcGGwz1");
    if(RpcGGwz2)accept("RpcGGwz2");
    if(RpcGGwz3)accept("RpcGGwz3");
    if(RpcGGslep1)accept("RpcGGslep1");
    if(RpcGGslep2)accept("RpcGGslep2");
    if(RpcGGslep3)accept("RpcGGslep3");
    if(RpvUDD1b)accept("RpvUDD1b");
    if(RpvUDD2b)accept("RpvUDD2b");
    if(RpvUDDge2b)accept("RpvUDDge2b");
    if(RpvUDDge3b)accept("RpvUDDge3b");
    if(RpvLQD)accept("RpvLQD");
    if(RpcSSslep1_Excl)accept("RpcSSslep1_Excl");
    if(RpcSSslep1)accept("RpcSSslep1");
    if(RpcSSslep2)accept("RpcSSslep2");
    if(RpcSSslep3)accept("RpcSSslep3");
    if(RpcSSslep4)accept("RpcSSslep4");
    if(RpcSSwz1)accept("RpcSSwz1");
    if(RpcSSwz2)accept("RpcSSwz2");
    if(RpcSSwz3)accept("RpcSSwz3");
    if(RpcSSwz4)accept("RpcSSwz4");
	if(InclusiveRpvUDD1b){accept("InclusiveRpvUDD1b");}

//	if()accept("");

if(true){
		accept("preselection");
    ntupVar("CRWZ2j",CRWZ2j);
    ntupVar("VRWZ4j",VRWZ4j);
    ntupVar("VRWZ6j",VRWZ6j);
    ntupVar("VRTTV",VRTTV);
    ntupVar("VRTTW",VRTTW);
    ntupVar("VRTTW3j",VRTTW3j);
    ntupVar("miupair",miupair);
    ntupVar("emiupair",emiupair);
	ntupVar("Pt_subjet",Pt_subjet);
	ntupVar("Pt_jet",Pt_jet);
	ntupVar("Pt_l",Pt_l);
	ntupVar("Pt_subl",Pt_subl);
	ntupVar("Pt_thirdl",Pt_thirdl );
	ntupVar("met",met);
	ntupVar("meff",meff );
	ntupVar("nBjets20",nBjets20 );
    ntupVar("nJets20",nJets20);
	ntupVar("nJets25",nJets25);
    ntupVar("nJets35",nJets35);
	ntupVar("nJets40",nJets40);
	ntupVar("nJets50",nJets50);
    ntupVar("nJets60",nJets60);
	ntupVar("isSS30",isSS30);
	ntupVar("Pt_Bjet",Pt_Bjet );
	ntupVar("ht_bjet", ht_bjet);
	ntupVar("ht_lept",ht_lept );
	ntupVar("ht_jet",ht_jet );
	ntupVar("dRll",dRll);
	ntupVar("dRl1j",dRl1j );
	ntupVar("nSigLep",nSigLep );
	ntupVar("NlepBL",NlepBL );
	ntupVar("mjj",mjj );
    ntupVar("mll",mll);
	ntupVar("mSFOS",mSFOS );
	ntupVar("mt_min",mt_min );
	ntupVar("met_Sig",metSignificance );
    ntupVar("BlindingVeto",BlindingVeto);
    ntupVar("dPhiLLmet",dPhiLLmet);

		fill("met",met);
		fill("ht_jet",ht_jet);
		fill("Pt_l",Pt_l);
		fill("Pt_subl",Pt_subl);
		fill("ht_bjet",ht_bjet);
		fill("ht_lept",ht_lept);
		fill("dRl1j",dRl1j);
		fill("dRll",dRll);
		fill("mSFOS",mSFOS);
		fill("mll",mll);
		fill("meff",meff);
		fill("nJets40",nJets40);
		fill("nSigLep",nSigLep);
		fill("mjj",mjj);
		fill("Pt_jet",Pt_jet);
		fill("Pt_subjet",Pt_subjet);
		fill("Pt_Bjet",Pt_Bjet);
		fill("nJets25",nJets25);
		fill("nBjets20",nBjets20);
		fill("mt_min",mt_min);
        fill("dPhiLLmet",dPhiLLmet);
	}

	return;
}


